#import "RNGeoLocationAMap.h"

#import <AMapFoundationKit/AMapFoundationKit.h>
#import <AMapLocationKit/AMapLocationKit.h>

@interface RNGeoLocationAMap () <AMapLocationManagerDelegate>

@end

@implementation RNGeoLocationAMap {
    AMapLocationManager *_manager;
}

RCT_EXPORT_MODULE()

RCT_REMAP_METHOD(init, initWithKey
                 : (NSString *)key
                 : (RCTPromiseResolveBlock)resolve
                 : (RCTPromiseRejectBlock)reject) {
  [AMapServices sharedServices].apiKey = key;
  if (!_manager) {
    _manager = [[AMapLocationManager alloc] init];
    _manager.delegate = self;
  }
  resolve(nil);
}

RCT_EXPORT_METHOD(startUpdatingLocation) { [_manager startUpdatingLocation]; }

RCT_EXPORT_METHOD(stopUpdatingLocation) { [_manager stopUpdatingLocation]; }

RCT_EXPORT_METHOD(setLocatingWithReGeocode : (BOOL)needAddress) {
  [_manager setLocatingWithReGeocode:needAddress];
}

RCT_EXPORT_METHOD(setAllowsBackgroundLocationUpdates : (BOOL)allowsBackground) {
  [_manager setAllowsBackgroundLocationUpdates:allowsBackground];
}

// 设定期望的定位精度。单位米
RCT_EXPORT_METHOD(setDesiredAccuracy : (int)meters) {
  [_manager setDesiredAccuracy:meters];
}

- (id)json:(CLLocation *)location reGeocode:(AMapLocationReGeocode *)reGeocode {
  if (reGeocode) {
    return @{
      @"errorCode": @(0),
      @"accuracy" : @(location.horizontalAccuracy),
      @"latitude" : @(location.coordinate.latitude),
      @"longitude" : @(location.coordinate.longitude),
      @"altitude" : @(location.altitude),
      @"speed" : @(location.speed),
      @"heading" : @(location.course),
      @"timestamp" : @(location.timestamp.timeIntervalSince1970 * 1000),
      @"address" : reGeocode.formattedAddress ? reGeocode.formattedAddress : @"",
      @"poiName" : reGeocode.POIName ? reGeocode.POIName : @"",
      @"country" : reGeocode.country ? reGeocode.country : @"",
      @"province" : reGeocode.province ? reGeocode.province : @"",
      @"city" : reGeocode.city ? reGeocode.city : @"",
      @"cityCode" : reGeocode.citycode ? reGeocode.citycode : @"",
      @"district" : reGeocode.district ? reGeocode.district : @"",
      @"street" : reGeocode.street ? reGeocode.street : @"",
      @"streetNumber" : reGeocode.number ? reGeocode.number : @"",
      @"adCode" : reGeocode.adcode ? reGeocode.adcode : @"",
    };
  } else {
    return @{
      @"errorCode": @(0),
      @"accuracy" : @(location.horizontalAccuracy),
      @"latitude" : @(location.coordinate.latitude),
      @"longitude" : @(location.coordinate.longitude),
      @"altitude" : @(location.altitude),
      @"speed" : @(location.speed),
      @"direction" : @(location.course),
      @"timestamp" : @(location.timestamp.timeIntervalSince1970 * 1000),
    };
  }
}

RCT_EXPORT_METHOD(requestLocationWithReGeocode : (BOOL)needReGeo) {
    // 单次定位，带逆地理（返回坐标和地址信息）,needReGeo = NO 则不会返回地址信息。
    [_manager requestLocationWithReGeocode:needReGeo completionBlock:^(CLLocation *location, AMapLocationReGeocode *reGeocode, NSError *error) {
        if (error) {
            [self sendEventWithName:@"onLocationChanged"
            body:@{
              @"errorCode" : @(error.code),
              @"errorInfo" : error.localizedDescription,
            }];
        }
        if (reGeocode) {
            id json = [self json:location reGeocode:reGeocode];
            [self sendEventWithName:@"onLocationChanged" body:json];
        }
    }];
}

- (void)amapLocationManager:(AMapLocationManager *)manager
          didUpdateLocation:(CLLocation *)location
                  reGeocode:(AMapLocationReGeocode *)reGeocode {
  id json = [self json:location reGeocode:reGeocode];
  [self sendEventWithName:@"onLocationChanged" body:json];
}

- (void)amapLocationManager:(AMapLocationManager *)manager
           didFailWithError:(NSError *)error {
  [self sendEventWithName:@"onLocationChanged"
                     body:@{
                       @"errorCode" : @(error.code),
                       @"errorInfo" : error.localizedDescription,
                     }];
}

- (void)amapLocationManager:(AMapLocationManager *)manager doRequireLocationAuth:(CLLocationManager *)locationManager{
  [locationManager requestAlwaysAuthorization];
}

- (NSArray<NSString *> *)supportedEvents {
  return @[ @"onLocationChanged" ];
}

@end
